<!DOCTYPE html>
<html>
    <head>
        <title>test-datatable</title>
        {{ catalog.render_assets() }}
    </head>
    <body>
        {% set columns= [
            {'name':'id', 'data':'id', 'title': 'ID'},
            {'name':'name', 'data':'name', 'title': 'Name'},
            {'name':'email', 'data':'email', 'title': 'Email'},
            {'name':'age', 'data':'age', 'title': 'Age'},
        ]%}
        <Datatable data={{ '/data.json' }} columns={{ columns }} />
    </body>
</html>
